function [] = equalize_audiofile( file_name )
%function [] = equalize_audiofile( file_name )
%
% Prepare equalizing example similar to Dave Moulton's Golden Ears.
%
% file_name: name of audio file to be processed, has to be .wav and 
%            file_name has to include the .wav extension.
%
% The function will write the processed file in the same directory like the
% original file. The file_name will be extended by the string '_eq'.
%
% Author: Jens Ahrens
%
% Open this script to comment/uncomment options.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  options  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%% frequency bands %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % 1/3 octaves
% freqs = [ 32 40 50 64 80 100 125 150 190 240 300 370 470 580 730 900 ...
%             1100 1400 1800 2200 2800 3500 4300 5500 6800 8500 11000 ...
%                 13000 16000 ];

% octaves
freqs = [ 32 64 125 250 500 1000 2000 4000 8000 16000 ]; % Hz

% % octaves in low-frequency range only
% freqs = [ 32 40 50 64 80 100 125 150 190 240 ];

% % octaves in mid-frequency range only
% freqs = [ 300 370 470 580 730 900 1100 1400 1800 ];

% % octaves in high-frequency range only
% freqs = [ 2200 2800 3500 4300 5500 6800 8500 11000 13000 16000 ];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%% bandwidth %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
band_width = 1; % in octaves
% band_width = 1/3; % in octaves

%%%%%%%%%%%%%%%%%%%%%%%%%% amplification %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
G = 12; % boost by G dB
%G = -12; % attenuate by |G| dB
boost_and_attenuation = false; % if true, both boost and attenuation will 
                               % be applied in a random order
                               
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%% end options  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% duration of sections in seconds
dur_pre_eq  = 5; 
dur_eq      = 7; 
dur_post_eq = 2;
dur_fade    = 0.5;

% check if .wav file
if ( ( size( file_name, 2 ) < 5 ) || ( ~strcmp( file_name( end - 3 : end ), '.wav' ) ) )      
    fprintf( '\nFile ''%s'' is not a .wav file. Skipping.\n\n', file_name );
    return;
else 
    fprintf( '\nLoading file ''%s'' ... ', file_name );
end


% read input signal
[ sig_in, fs, bits ] = wavread( file_name );

fprintf( 'Done.\n\n');
fprintf( 'Processing ... \n\n' );

max_sig_in     = max( abs( sig_in(:) ) ); 
no_of_channels = size( sig_in, 2 );
dur_sig        = size( sig_in, 1 );

% durations of sections in samples
dur_pre_eq  = round( dur_pre_eq  * fs );
dur_eq      = round( dur_eq      * fs );
dur_post_eq = round( dur_post_eq * fs );
dur_fade    = round( dur_fade     * fs );

% crossfade window
fades    = hann( 2 * dur_fade, 'periodic' );
fade_in  = fades( 1 : end/2 );
fade_out = fades( end/2 + 1 : end );

% allocate memory for output signal
sig_out = zeros( size( sig_in ) );

% open text file to save equalization parameters
fid = fopen( [ file_name( 1 : end - 4 )  '_eq.txt' ], 'wt' );

% write to command window
fprintf( 'Frequency   Bandwidth   Gain\n\n' );

% write to text file
fprintf( fid, 'Frequency   Bandwidth   Gain\n\n\n\n\n\n' );

% make sure that we get a different result each time we execute the script
rand( 'state', sum( 100*clock ) );

% counters for input signal samples
sample_index = 1;

while ( sample_index + dur_pre_eq + dur_eq + dur_post_eq < dur_sig )

    % new filter frequency
    random_integers = randperm( length( freqs ) );
    f_filt          = freqs( random_integers( 1 ) );
    bw              = f_filt * band_width;
    
    if ( boost_and_attenuation )
        G = G * sign( rand - .5 );
    end

    parameter_string = sprintf( '%9d %6.2f oct. %4d dB\n', f_filt, band_width, G );

    % write parameters to command window
    fprintf( parameter_string );

    % write parameters to file
    fprintf( fid, [ parameter_string '\n\n\n' ] );       

    % pre-eq phase
    sig_out( sample_index : sample_index + dur_pre_eq - 1, : ) = ...
                     sig_in( sample_index : sample_index + dur_pre_eq - 1, : );

    % hop further
    sample_index = sample_index + dur_pre_eq;
    
    % eq phase
    for channel = 1 : no_of_channels 
        
        processed_chunk = sig_in( sample_index : sample_index + dur_eq - 1, channel );
        
        % perform filtering
        processed_chunk = peakfilt( processed_chunk, f_filt, bw, G, fs );

        % crossfades
        processed_chunk( 1 : dur_fade ) = processed_chunk( 1 : dur_fade ) .* fade_in + ...
                                             sig_in( sample_index : sample_index + dur_fade - 1, channel ) .* fade_out;

        processed_chunk( dur_eq - dur_fade + 1 : dur_eq ) = processed_chunk( dur_eq - dur_fade + 1 : dur_eq ) .* fade_out + ...
                                             sig_in( sample_index + dur_eq - dur_fade : sample_index + dur_eq - 1, channel ) .* fade_in;

        % add processed chunk to output signal
        sig_out( sample_index : sample_index + dur_eq - 1, channel ) = processed_chunk;

    end    
    
    % hop further
    sample_index = sample_index + dur_eq;
    
    % post-eq phase
    sig_out( sample_index : sample_index + dur_post_eq - 1, : ) = ...
                    sig_in( sample_index : sample_index + dur_post_eq - 1, : );

    % hop further
    sample_index = sample_index + dur_post_eq;

end

% remainder of audio signal
sig_out( sample_index : end, : ) = sig_in( sample_index : end, : );

fclose( fid );

clear sig_in processed_chunk;

max_sig_out = max( abs( sig_out(:) ) );

% recover approximately the relative input level
sig_out = sig_out ./ max_sig_out .* max_sig_in;

fprintf( '\nWriting file to disk ... ' );

wavwrite( sig_out, fs, bits, [ file_name( 1 : end - 4 )  '_eq.wav' ] );

fprintf( 'Done.\n\n' );

end

function out = peakfilt( sig, fc, fb, G, fs )

% peakfilt              - Peak filter, second order
%
% out = peakfilt (sig,fc,fb,G,fs)
% 
% Input parameters:
% - sig:    input signal
% - G:      gain [dB]
% - fc:     cut-off frequency
% - fb:     bandwidth
% - fs:     sampling frequency
% Output parameter:
% - out:    output signal (column vector)
% 
% written by Piotr Majdak
% obtained from http://piotr.majdak.com/download/mlib/index.php

b=(tan(pi*fb/fs)-1)/(tan(pi*fb/fs)+1);
c=-cos(2*pi*fc/fs);
H=10^(G/20)-1;
H=H/2;

len=length(sig)+2;
y=zeros(len,1);
yh=zeros(len,1);
x=zeros(len,1);
x(3:len)=sig;

for n=3:len
    yh(n) = -b*x(n)+c*(1-b)*x(n-1)+x(n-2)-c*(1-b)*yh(n-1)+b*yh(n-2);
    y(n)= H*(x(n)-yh(n))+x(n);
end

out=y(3:end);

end

